#!/usr/bin/ruby
#
# Author:
#   john [at] tinternet org uk
#
# Desc:
#   module to perform checks/actions against authentication related stuff such as
#   loggin in or listing sessions
#
# Lisence:
#   see vim25.rb for lisence and copyright information
#

module Auth

  include Vim25

  #
  #
  # this class describes the Password class, which is used to create a stack of
  # passwords which are threadsafe
  class Password

    # takes and array of passwords
    def initialize(passwords)
      # there is no top method, so i'll have to use pop. So i'll reverse the list of passwords first
      # so that it goes a-z rather than z-a
      @passwords = passwords.reverse
    end

    # returns the next password in the list
    def pop
      return @passwords.pop
    end

    # returns whether the password object is empty - i.e. all passwords have been used
    def empty
      if @passwords.size == 0 then
        return true
      else
        return false
      end
    end # end empty
  end # end Password class



  #
  #
  # log into the web service
  def login(username,password)
    sessref = ManagedObjectReference.new(@servicecontent.returnval.sessionManager)
    sessref.xmlattr_type = "SessionManager"

    # create a new object to represent a login
    loginReq = LoginRequestType.new(sessref,username,password,"en-gb")

    # make login request. return the LoginResponse object
    begin
      session = @driver.login(loginReq)
      #puts "Log in successful - session ID:\t "+session.returnval.key
    rescue
      #puts "Login Failed: #{$!}"
    end

    # return the LoginResponse object
    return session

  end # end login method


  #
  #
  # bruteforce login
  # - currently limited to only bruteforcing a single username
  # - defaults to 8 threads
  def bruteforceLogin(username,dictionary)

    require 'pty'
    require 'monitor'

    # read all passwords into an array
    passwords = [] 

    begin
      passwords = File.readlines(dictionary)
    rescue
      return "ERROR: password file does not exist or is not readable\n\n"
      exit
    end

    # password object where passwords can be retrieved from
    pass = Password.new(passwords)

    sessref = ManagedObjectReference.new(@servicecontent.returnval.sessionManager)
    sessref.xmlattr_type = "SessionManager"

    def doLogin(username,passwd,s_ref)
      # create a new object to represent a login
      loginReq = LoginRequestType.new(s_ref,username,passwd,"en-gb")

      begin
        session = @driver.login(loginReq)
      rescue
      end

      puts "Trying: #{username} #{passwd}"

      if session != nil
        puts "** SUCCESS : #{username} / #{passwd}"
      end
    end #end doLogin


    lock = Monitor.new
    threads = []

    for i in 1..8 do
      threads << Thread.new(i){ |threadID|
        puts "> starting thread #{i}"
        while lock.synchronize{!pass.empty} do
          begin
            doLogin(username,lock.synchronize{pass.pop.chomp},sessref)
          rescue
          end
        end
      }
    end
    sleep(5)
    threads.each{|thr| thr.join}

  end


  #
  #
  # log out of the current session
  def logout
    sessref = ManagedObjectReference.new(@servicecontent.returnval.sessionManager)
    sessref.xmlattr_type = "SessionManager"
    logoutReq = LogoutRequestType.new(sessref)
    #puts "Logged out"
    return @driver.logout(logoutReq)
  end

  #
  #
  # get a list of active sessions
  def getSessions
    sessionInfo = retrieveProperty(@servicecontent.returnval.sessionManager,
                     true,
                     "SessionManager",
                     "sessionList",
                     false)

    sessions_ary = []
    sessionInfo[0].propSet[0].val.each do |userSession|
      sessions_ary << userSession.key
    end

    return sessions_ary

  end # end getSessions method

  #
  #
  # list the user accounts on the host
  def listUsers

      usergroupreply = getUsers

      users_ary = []
      usergroupreply.each do |user|
        # TODO: may be worth creating a user object, or returning the entire user
        #       object so that the aditional properties can be extracted as desired
        users_ary << "#{user.principal}/#{user.fullName}/#{user.id}/#{user.shellAccess}\n"
      end
      return users_ary

  end # end listUsers

  #
  #
  # lists all non standard user accounts
  # NOTE: these users differ on ESX and ESXi hosts, so a check for the technology
  #       type needs implementing and an ESXi list of standard users needs adding
  def getNonStandardUsers

    standardUsers_ary = ["root",
                         "bin",
                         "daemon",
                         "adm",
                         "lp",
                         "sync",
                         "shutdown",
                         "halt",
                         "mail",
                         "news",
                         "uucp",
                         "operator",
                         "gopher",
                         "ftp",
                         "nobody",
                         "nscd",
                         "vcsa",
                         "ntp",
                         "sshd",
                         "rpc",
                         "rpcuser",
                         "nfsnobody",
                         "pcap",
                         "rpm",
                         "vimuser"]

    # get current users on system
    currentUsers_ary = []
    users = getUsers
    users.each do |user|
      currentUsers_ary << user.principal
    end

    # compare current users to default users

    # if both arrays have the same content then just return
    if standardUsers_ary.eql?(currentUsers_ary)
      return
    end

    # subtract both arrays from each other
    nonstandardUsers_ary = currentUsers_ary - standardUsers_ary
    return nonstandardUsers_ary

  end # end getNonStandardUsers


  #
  #
  # list users with shell
  def listShellUsers

    users = getUsers
    shellUsers = []
    users.each do |user|
      if user.shellAccess
        shellUsers << user.principal+"\n"
      end
    end

    return shellUsers

  end # end listShellUsers


  private
  # retrieves users from the server - used to support some of the methods above rather
  # than replicating this method in mulitple other methods
  def getUsers
      servcont = ManagedObjectReference.new(@servicecontent.returnval.userDirectory)
      servcont.xmlattr_type = "UserDirectory"
      usergroupsreq = RetrieveUserGroupsRequestType.new(servcont)
      usergroupsreq.searchStr = '' # blank matches all users
      usergroupsreq.exactMatch = false
      usergroupsreq.findUsers = true
      usergroupsreq.findGroups = false
      return @driver.retrieveUserGroups(usergroupsreq)
  end


end # end Auth module
