#!/usr/bin/ruby
#
# Author:
#   john [at] tinternet org uk
#
# Desc:
#   module to perform checks/actions against network related objects
#
# Lisence:
#   see vim25.rb for lisence and copyright information

module Network

  include Vim25

  #
  #
  # get a list of networks from the portgroupInfo - group networks by their
  # switch
  def listNetworks

    portGroupInfo = getPortGroupInfo
    portGroupInfo.each do |item|
      puts item.class
      item.propSet.each do |itemel|
        puts "\t"+itemel.class.to_s+" - "+itemel.val.class.to_s
      end
    end

  end # end list networks



  ########### DNS related methods #################
  #
  #
  # obtains a list of the DNS servers
  def listDNSServers
    return getDNSConfig[0].propSet[0].val.address
  end # end listDNSServers

  #
  #
  # finds out whether DHCP is used
  def dhcpEnabled?
    return getDNSConfig[0].propSet[0].val.dhcp
  end

  #
  #
  # gives the search domain for the dns
  def getSearchDomain
    return getDNSConfig[0].propSet[0].val.searchDomain
  end

  #
  #
  # gives the systems hostname
  def getHostname
    return getDNSConfig[0].propSet[0].val.hostName
  end
  #
  ########### End DNS related methods ##############

  #
  #
  # gets information about the console Nic
  def getConsoleNicInfo
    info = getConsoleVnicInfo
    portGroup = info[0].propSet[0].val[0].portgroup
    device = info[0].propSet[0].val[0].device
    mac = info[0].propSet[0].val[0].spec.mac
    ipaddr = info[0].propSet[0].val[0].spec.ip.ipAddress
    subnet = info[0].propSet[0].val[0].spec.ip.subnetMask
    if info[0].propSet[0].val[0].spec.ip.dhcp
      dhcp = "DHCP"
    else
      dhcp = "no DHCP"
    end

    return portGroup+"/"+device+"/"+mac+"/"+ipaddr+"/"+subnet+"/"+dhcp

  end

  #
  #
  # lists all physical nics on the box
  def listPnics
    info = getNetConfig("pnic")
    nics = []
    info[0].propSet[0].val.each do |nic|
      nics << nic.device
    end
    return nics
  end

  #
  #
  # list all portgroups
  def listPortgroups
    pgi = getNetConfig("portgroup")
    pg_ary = []
    pgi[0].propSet[0].val.each do |portgroup|
      pg_ary << portgroup.spec.vswitchName+"("+portgroup.spec.name+")_vlan("+portgroup.spec.vlanId.to_s+")"
    end
    return pg_ary
  end

  #
  #
  # check interface settings for forged transmits etc. for both switches and
  # for portgroups
  def checkInterfaceSecurity

    vsi = getVswitchInfo
    vs_ary = []
    vsi[0].propSet[0].val.each do |vswitch|
      vs_ary << vswitch.name+"("+vswitch.spec.bridge.nicDevice.to_s+").allowPromiscuous =\t"+vswitch.spec.policy.security.allowPromiscuous.to_s
      vs_ary << vswitch.name+"("+vswitch.spec.bridge.nicDevice.to_s+").forgedTransmits =\t"+vswitch.spec.policy.security.forgedTransmits.to_s
      vs_ary << vswitch.name+"("+vswitch.spec.bridge.nicDevice.to_s+").macChanges =\t\t"+vswitch.spec.policy.security.macChanges.to_s
    end

    pgi = getNetConfig("portgroup")
    pg_ary = []
    pgi[0].propSet[0].val.each do |portgroup|
      pg_ary << portgroup.spec.vswitchName+"."+portgroup.spec.name+"."+"allowPromiscuous =\t"+portgroup.spec.policy.security.allowPromiscuous.to_s
      pg_ary << portgroup.spec.vswitchName+"."+portgroup.spec.name+"."+"forgedTransmits =\t"+portgroup.spec.policy.security.forgedTransmits.to_s
      pg_ary << portgroup.spec.vswitchName+"."+portgroup.spec.name+"."+"macChanges =\t\t"+portgroup.spec.policy.security.macChanges.to_s
    end
    return vs_ary+pg_ary
  end # end checkInterfaceSecurity


  private
  #
  #
  # retrieves the specified info from the HostNetworkConfig managed object ref.
  def getNetConfig(config)
    return retrieveProperty(@configManager[0].propSet[4].val.networkSystem,
                     false,
                     "HostNetworkSystem",
                     "networkConfig.#{config}",
                     false)
  end # end getNetConfig

  #
  #
  # get portGroup information
  def getPortGroupInfo
    getNetConfig("portgroup")
  end # end getPortGroupInfo

  #
  #
  # getDNSConfig information
  def getDNSConfig
    getNetConfig("dnsConfig")
  end # end getDNSConfig

  #
  #
  # get console NIC information
  def getConsoleVnicInfo
    getNetConfig("consoleVnic") # consoleVnic is unset with ESXi
  end # end getDNSConfig

  #
  #
  # get virtual switch information
  def getVswitchInfo
    getNetConfig("vswitch")
  end # end get virtual switch info



end # end Network module
