#!/usr/bin/ruby
#
# Author:
#   john [at] tinternet org uk
#
# Desc:
#   module to perform retrieve info / perform actions against VM's
#
# Lisence:
#   see vim25.rb for lisence and copyright information

module VirtualMachine

  #
  #
  # returns a virtual machine as maintained by the ESX server
  def getVM(vm_id)
    # the virtual machines are held in an array - we need a mapping
    # between their position and their value this is what the index call on 
    # vm_ary determines
    vm_ary = @datastore[0].propSet[7].val
    return retrieveProperty(@datastore[0].propSet[7].val[vm_ary.index(vm_id)],
                            true,
                            "VirtualMachine",
                            "name",
                            true)[0].propSet
  end # end getVM


  #
  #
  # obtain a list of virtual machines running on the host
  def listVMIDs

    vms_ary = []

    #puts "there are #{@conninfo.returnval.vm.size} VMs on this host"
    for i in 0..@conninfo.returnval.vm.size-1
      vms_ary << @conninfo.returnval.vm[i].vm #.vm
    end

    return vms_ary
  end # end listVMs method


  #
  #
  # obtain a list of virtual machines running on the host
  def listVMs

    vms_ary = []

    #puts "there are #{@conninfo.returnval.vm.size} VMs on this host"
    for i in 0..@conninfo.returnval.vm.size-1
      vms_ary << "["+@conninfo.returnval.vm[i].vm+"] "+
                 @conninfo.returnval.vm[i].config.name+" "+
                 @conninfo.returnval.vm[i].config.guestFullName+" - "+
                 @conninfo.returnval.vm[i].config.guestId+" - "+
                 @conninfo.returnval.vm[i].config.uuid
    end

    # if there are no vms then just return the empty array
    if vms_ary.size == 0
      return []
    end

    return vms_ary
  end # end listVMs method


  #
  #
  # gets a friendly name for a virtual machine given its name
  def getFriendlyName(vmId)

    @conninfo.returnval.vm.each do |vminfo|
      if vminfo.vm.eql?(vmId)
        return vminfo.config.name
      end
    end # end each do

    return nil
  end # end getFriendlyName


  # TODO: tidy this method up so it reads sensible settings from a config file
  #       NOTE: it may actually be better to grab the config file and parse it that way
  #             instead of doing it how it is here since settings are dotted all over
  #             the place
  #
  # checks to see if copying and pasting and cdrom are available from host to guest
  #
  # TODO: ALSO ADD SOME CHEKCS RE LOG FILES
  #
  def checkConfig

    vms = listVMIDs
    results = []

    vms.each do |vm_id|
      config = getVMConfig(vm_id)

      results << "\n____[ #{vm_id} : #{getFriendlyName(vm_id)} ]____"

      copy_disable = false
      paste_disable = false
      guiopts_disable = false
      device_disable = false

      # this needs to be read from the some config file at some point
      # check for copy enabled
      config.val.each do |pair|

        case (pair.key)
        when "isolation.tools.copy.disable"
          results << pair.key+" = "+pair.value#+" \t\t- this should be set to 'true'"
          copy_disable = true
        when "isolation.tools.paste.disable"
          results << pair.key+" = "+pair.value#+" \t\t -this should be set to 'true'"
          paste_disable = true
        when "isolation.tools.setGUIOptions.enable"
          results << pair.key+" = "+pair.value#+" \t- this should be set to 'false'"
          guiopts_disable = true
        when "isolation.tools.connectable.disable"
          results << pair.key+" = "+pair.value#+" \t- this should be set to 'true'"
          device_disable = true
        end # end case
      end

      # flag anything which is missing
      if !copy_disable
        results << "isolation.tools.copy.disable = UNSET" # \t\t- this should be set to 'true'"
      end
      if !paste_disable
        results << "isolation.tools.paste.disable = UNSET" # \t\t- this should be set to 'true'"
      end
      if !guiopts_disable
        results << "isolation.tools.setGUIOptions.enable = UNSET" # \t- this should be set to 'false'"
      end
      if !device_disable
        results << "isolation.tools.connectable.disable = UNSET" # \t- this should be set to 'true'"
      end

    end # end vms.each

    return results

  end # end checkConfig


  #
  #
  # check the total memory allocated for VMs and compare it to total available RAM
  # TODO
  def checkMemUseage

  end # end checkMemUseage


    private


    #
    #
    # returns the configuration of a virtual machine
    def getVMConfig(vm_id)
      # the virtual machines are held in an array - we need a mapping
      # between their position and their value this is what the index call on 
      # vm_ary determines
      vm_ary = @datastore[0].propSet[7].val
      return retrieveProperty(@datastore[0].propSet[7].val[vm_ary.index(vm_id)],
                              true,
                              "VirtualMachine",
                              "config.extraConfig",
                              false)[0].propSet[0] # returns array of key value pairs
    end # end getVM













######################### end module ##################

######################### class #######################




  #
  #
  # this class describes a virtual machine and its associated methods. it makes sense
  # to encapsulate a virtual machine within a virtual machine object and give it its
  # various methods 
  #
  # TODO: find out what is returned by the listVMs method and see whether it would
  #       be better to use that object, and/or extend it instead of writing soemthing
  #       from scratch.
  class VirtualMachineObj

    def initialize

    end

    #
    #
    # power on VM
    def powerOn

    end # end powerOn method

    #
    #
    # power off the virtual machine
    def powerOff

    end # end powerOff method

  end # end VirtualMachine class


end # end VirtualMachine module